/*---------------------------------------------------------------------------*
  Copyright 2010-2011 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#ifndef _WKSUP_GFX_SHADER_H_
#define _WKSUP_GFX_SHADER_H_

#include <wksup/wksupGfxTypes.h>
#include <wksup/wksupGfxShader.h>

#ifdef __cplusplus
extern "C" {
#endif

/// @addtogroup wksupShader
/// @{

/// \brief The structure for vertex/pixel samplers
///
/// Each member map to each sampler number.
///
typedef struct _wksupGfxSamplers
{
    /// Current number of sampler
    u32             count;

    /// The locations in the shader.
    u32             location[GX2_MAX_SAMPLERS];
} wksupGfxSamplers;


/// \brief The structure for vertex uniforms
///
/// Each member map to each uniform number.
///
typedef struct _wksupGfxUniformsVS
{
    /// Current number of uniforms
    u32             count;

    /// The locations in the vertex shader.
    u32             location[GX2_MAX_VS_UNIFORM_VARS];
} wksupGfxUniformsVS;


/// \brief The structure for pixel uniforms
///
/// Each member map to each uniform number.
///
typedef struct _wksupGfxUniformsPS
{
    /// Current number of uniforms
    u32             count;

    /// The locations in the pixel shader.
    u32             location[GX2_MAX_PS_UNIFORM_VARS];
} wksupGfxUniformsPS;

/// \brief The structure for vertex uniform blocks
///
/// Each member map to each uniform number.
///
typedef struct _wksupGfxUniformBlocks
{
    /// Current number of uniform blocks
    u32             count;

    /// The locations in the shader.
    u32             location[GX2_MAX_UNIFORM_BLOCKS];

    /// The size of uniform blocks
    u32             size[GX2_MAX_UNIFORM_BLOCKS];

} wksupGfxUniformBlocks;

/// \brief The structure for shader data
///
/// Includes members to setup shader
///
typedef struct _wksupGfxShader
{
    /// Current number of attributes
    u32               attribCount;

    /// The attribue streams are used to bind the variables used by the vertex shader with the fetch shader.
    GX2AttribStream   attribs[GX2_MAX_ATTRIB_BUFFERS];

    /// The structure holds vertex shader samplers locations
    wksupGfxSamplers   samplersVS;

    /// The structure holds pixel shader samplers locations
    wksupGfxSamplers   samplersPS;

    /// The structure holds geometry shader samplers locations
    wksupGfxSamplers   samplersGS;

    /// The structure holds vertex shader uniforms locations
    wksupGfxUniformsVS uniformsVS;

    /// The structure holds pixel shader uniforms locations
    wksupGfxUniformsPS uniformsPS;

    // GS is UB only

    /// The structure holds vertex shader uniform blocks location and size
    wksupGfxUniformBlocks uniformBlocksVS;

    /// The structure holds pixel shader uniform blocks location and size
    wksupGfxUniformBlocks uniformBlocksPS;

    /// The structure holds geometry shader uniform blocks location and size
    wksupGfxUniformBlocks uniformBlocksGS;

    /// The vertex shader structure pointers which are loaded from the GSH file
    GX2VertexShader *pVertexShader;

    /// The pixel shader structure pointers which are loaded from the GSH file
    GX2PixelShader  *pPixelShader;

    /// The geometry shader structure pointers which are loaded from the GSH file
    GX2GeometryShader  *pGeometryShader;

    /// The fetch shader which is based on the given attribute stream list.
    GX2FetchShader  fetchShader;

    /// An aligned, user-allocated buffer for the fetch shader binary code.
    void * pFetchShaderBuffer;
} wksupGfxShader;

//
// Functions
//

/// \brief Checks number of shader location
///
/// \param location number of shader location
/// \return result (TRUE or FALSE)
///
inline BOOL wksupGfxCheckShaderLocation(u32 location)
{
    if(location == (u32)(-1))
    {
        OSReport("Warning : Couldn't find the correct location.\n");
        return FALSE;
    }

    return TRUE;
}

/// \brief Loads shaders from file buffer (.gsh)
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param index   Index of shader in the file buffer
/// \param pData   Pointer to the file buffer
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxLoadShaders(wksupGfxShader *pShader, u32 index, const void *pData);
BOOL wksupGfxLoadShadersUsingHeap(wksupGfxShader *pShader, u32 index, const void *pData, signed heap);

/// \brief Sets attribute menbers to wksupGfxShader structure and inisialize stream
///
/// \param pShader       Pointer to wksupGfxShader structure
/// \param name          Pointer to name of attribute (e.g a_position, a_color etc)
/// \param bufferIndex   Index of attribute buffer
/// \param offset        Offset value for attribute data
/// \param format        Attribute format
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxInitShaderAttribute(wksupGfxShader *pShader, const char *name,
                                u32 bufferIndex, u32 offset, GX2AttribFormat format);

/// \brief Gets vertex shader sampler location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of sampler (e.g s_texture etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetVertexShaderSamplerLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets pixel shader sampler location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of sampler (e.g s_texture etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetPixelShaderSamplerLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets geometry shader sampler location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of sampler (e.g s_texture etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetGeometryShaderSamplerLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets vertex shader uniform location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of uniform (e.g u_scale etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetVertexShaderUniformLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets pixel shader uniform location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of uniform (e.g u_scale etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetPixelShaderUniformLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets vertex shader uniform block location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of uniform block (e.g ub_staticBlock etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetVertexShaderUniformBlockLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets pixel shader uniform block location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of uniform block (e.g ub_dynamicBlock etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetPixelShaderUniformBlockLocation(wksupGfxShader *pShader, const char *name);

/// \brief Gets geometry shader uniform block location and sets to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
/// \param name    Pointer to name of uniform block (e.g ub_dynamicBlock etc)
/// \return result (TRUE or FALSE)
///
BOOL wksupGfxGetGeometryShaderUniformBlockLocation(wksupGfxShader *pShader, const char *name);

/// \brief Sets fetch shader with attrib stream data to wksupGfxShader structure
///
/// \param pShader Pointer to wksupGfxShader structure
///
BOOL wksupGfxInitFetchShader(wksupGfxShader *pShader);
BOOL wksupGfxInitFetchShaderUsingHeap(wksupGfxShader *pShader, signed heap);

/// \brief Free shaders (Vertex, Pixel, Geometry and Fetch shader )
///
/// \param pShader Pointer to wksupGfxShader structure
///
BOOL wksupGfxFreeShaders(wksupGfxShader *pShader);

/// @}

#ifdef __cplusplus
}
#endif

#endif // _WKSUP_GFX_SHADER_H_
